VERSION 5.00
Begin VB.Form frmMain 
   BorderStyle     =   3  'Fixed Dialog
   Caption         =   "DirectX-Graphics: Lesson 01"
   ClientHeight    =   3195
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   4680
   ControlBox      =   0   'False
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   213
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   312
   ShowInTaskbar   =   0   'False
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'###############################
'
'           Title: DirectX-Graphics Article 02
'           Desc: An introduction to basic 3D geometry
'           Written by: Jack Hoxley
'           Date: march 13th 2001
'           Contact: Jollyjeffers@Greenonions.netscapeonline.co.uk
'                       www.vbexplorer.com/directx4vb/
'           NB: Sample/Article was written for GameDev.Net
'
'###############################

Dim Dx As DirectX8 'The master Object, everything comes from here
Dim D3D As Direct3D8 'This controls all things 3D
Dim D3DDevice As Direct3DDevice8 'This actually represents the hardware doing the rendering
Dim bRunning As Boolean 'Controls whether the program is running or not...

Const FVF_TLVERTEX = (D3DFVF_XYZRHW Or D3DFVF_TEX1 Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR)
Const FVF_LVERTEX = (D3DFVF_XYZ Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR Or D3DFVF_TEX1)
Const FVF_VERTEX = (D3DFVF_XYZ Or D3DFVF_NORMAL Or D3DFVF_TEX1)

Private Type TLVERTEX
    X As Single
    Y As Single
    Z As Single
    rhw As Single
    Color As Long
    Specular As Long
    tU As Single
    tV As Single
End Type

Private Type LITVERTEX
    X As Single
    Y As Single
    Z As Single
    Color As Long
    Specular As Long
    tU As Single
    tV As Single
End Type

Private Type VERTEX
    X As Single
    Y As Single
    Z As Single
    nx As Single
    ny As Single
    nz As Single
    tU As Single
    tV As Single
End Type

Dim TriVert(0 To 2) As TLVERTEX '//we require 3 vertices to make a triangle...

Dim CubeVerts(0 To 35) As LITVERTEX '//for our 3D cube

Dim vbCube As Direct3DVertexBuffer8 '//The vertex buffer for our cube.

Dim vbCubeIdx As Direct3DVertexBuffer8
Dim ibCube As Direct3DIndexBuffer8
Dim vList(0 To 7) As LITVERTEX 'the 8 vertices required
Dim iList(0 To 35) As Integer 'the 36 indices required (note that the number is the same as the vertex count in the previous version).

Dim matProj As D3DMATRIX
Dim matView As D3DMATRIX
Dim matWorld As D3DMATRIX

Const PI = 3.14159
Const RAD = PI / 180
Const DEG = 180 / PI




'// Initialise : This procedure kick starts the whole process.
'// It'll return true for success, false if there was an error.
Public Function Initialise() As Boolean
On Error GoTo ErrHandler:

Dim DispMode As D3DDISPLAYMODE '//Describes our Display Mode
Dim tmpDispMode As D3DDISPLAYMODE '//used during the enumeration of avail. modes
Dim D3DWindow As D3DPRESENT_PARAMETERS '//Describes our Viewport
Dim I As Long '//so we can loop through the avail. display modes

Set Dx = New DirectX8  '//Create our Master Object
Set D3D = Dx.Direct3DCreate() '//Let our Master Object create the Direct3D Interface

'##################
'##  FULLSCREEN MODE ##
'#################
For I = 0 To D3D.GetAdapterModeCount(0) - 1 'primary adapter
    D3D.EnumAdapterModes 0, I, tmpDispMode
    Debug.Print tmpDispMode.Width & "x" & tmpDispMode.Height & "x" & tmpDispMode.Format
Next I

DispMode.Format = CheckDisplayMode(640, 480, 32)
Debug.Print "CHECKDISP(640,480,32) = ", DispMode.Format, D3DFMT_UNKNOWN
If DispMode.Format > D3DFMT_UNKNOWN Then
    '640x480x32 is supported
    Debug.Print "USING 640x480x32 Display Mode"
    DispMode.Width = 640: DispMode.Height = 480
Else
    DispMode.Format = CheckDisplayMode(640, 480, 16)
    If DispMode.Format > D3DFMT_UNKNOWN Then
        '640x480x16 is supported
        Debug.Print "USING 640x480x16 Display Mode"
        DispMode.Width = 640: DispMode.Height = 480
    Else
        'hmm, neither are supported. oh well...
        MsgBox "Your hardware does not appear to support" _
                     & " 640x480 display modes in either 16 bit or 32 bit modes. Exiting" _
                    , vbInformation, "Error"
        Unload Me
        End
        
    End If
End If

D3DWindow.BackBufferCount = 1
D3DWindow.BackBufferFormat = DispMode.Format
D3DWindow.BackBufferWidth = DispMode.Width
D3DWindow.BackBufferHeight = DispMode.Height
D3DWindow.hDeviceWindow = frmMain.hWnd
D3DWindow.AutoDepthStencilFormat = D3DFMT_D16
D3DWindow.EnableAutoDepthStencil = 1
D3DWindow.SwapEffect = D3DSWAPEFFECT_COPY_VSYNC

'############################
'## CHECK THE DEVICE CAPABILITIES ##
'###########################
Dim DevCaps As D3DCAPS8
On Local Error Resume Next
    D3D.GetDeviceCaps D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DevCaps
If Err.Number = D3DERR_INVALIDDEVICE Then
    'We couldn't get data from the hardware device - probably doesn't exist...
    D3D.GetDeviceCaps D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, DevCaps
    Err.Clear '//Remove the error value..
End If


'//For Hardware vertex processing:
If (DevCaps.DevCaps And D3DDEVCAPS_HWTRANSFORMANDLIGHT) Then
    Debug.Print "Hardware Transform and lighting supported"
    'D3DCREATE_HARDWARE_VERTEXPROCESSING should be used in the CreateDevice call
Else
    Debug.Print "Hardware Transform and lighting is not supported"
End If

'//For Pure Device processing:
If (DevCaps.DevCaps And D3DDEVCAPS_PUREDEVICE) Then
    Debug.Print "Pure Device is supported."
    'D3DCREATE_PUREDEVICE should be used in the CreateDevice call
Else
    Debug.Print "Pure device is not supported"
End If

'//This line creates a device that uses a hardware device if possible; software vertex processing and uses the form as it's target
'//See the lesson text for more information on this line...
Set D3DDevice = D3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, _
                                                    frmMain.hWnd, _
                                                    D3DCREATE_SOFTWARE_VERTEXPROCESSING, _
                                                        D3DWindow)
                                                        
'//Configure the rendering device
D3DDevice.SetVertexShader FVF_LVERTEX '//Tell it what type of vertex we are using
D3DDevice.SetRenderState D3DRS_LIGHTING, 0  '//Disable lighting.
D3DDevice.SetRenderState D3DRS_ZENABLE, 1
D3DDevice.SetRenderState D3DRS_CULLMODE, D3DCULL_NONE

'//configure the world matrices

'//1. The World Matrix
D3DXMatrixIdentity matWorld
D3DDevice.SetTransform D3DTS_WORLD, matWorld 'commit this matrix to the device

'//2. The View Matrix
D3DXMatrixLookAtLH matView, MakeVector(0, 5, 2), MakeVector(0, 0, 0), MakeVector(0, 1, 0)
D3DDevice.SetTransform D3DTS_VIEW, matView

'//3. The projection Matrix
D3DXMatrixPerspectiveFovLH matProj, PI / 3, 1, 0.1, 75
D3DDevice.SetTransform D3DTS_PROJECTION, matProj

InitialiseGeometry '//Setup the vertices

Initialise = True '//We succeeded
Exit Function

ErrHandler:
Debug.Print "Error Number Returned: " & Err.Number, Err.Description
Initialise = False
End Function

Public Sub Render()
'//1. We need to clear the render device before we can draw anything
'       This must always happen before you start rendering stuff...
D3DDevice.Clear 0, ByVal 0, D3DCLEAR_TARGET Or D3DCLEAR_ZBUFFER, &H8080CC, 1#, 0
'the hexidecimal value in the middle is the same as when you're using colours in HTML - if you're familiar
'with that.

'//2. Next we would render everything.

D3DDevice.BeginScene
    '##RENDERING METHOD 1##
    'D3DDevice.DrawPrimitiveUP D3DPT_TRIANGLELIST, 12, CubeVerts(0), Len(CubeVerts(0))
    
    '##RENDERING METHOD 2##
    'D3DDevice.SetStreamSource 0, vbCube, Len(CubeVerts(0))
    'D3DDevice.DrawPrimitive D3DPT_TRIANGLELIST, 0, 12
    
    '##RENDERING METHOD 3##
    'D3DDevice.SetStreamSource 0, vbCubeIdx, Len(vList(0))
    'D3DDevice.SetIndices ibCube, 0
    'D3DDevice.DrawIndexedPrimitive D3DPT_TRIANGLELIST, 0, 36, 0, 12
    
    '##RENDERING METHOD 4##
    D3DDevice.DrawIndexedPrimitiveUP D3DPT_TRIANGLELIST, 0, 8, 12, iList(0), D3DFMT_INDEX16, vList(0), Len(vList(0))
D3DDevice.EndScene

'//3. Update the frame to the screen...
'       This is the same as the Primary.Flip method as used in DirectX 7
'       These values below should work for almost all cases...
D3DDevice.Present ByVal 0, ByVal 0, 0, ByVal 0
End Sub

Private Sub InitialiseGeometry()
'//0. Any Variables

'//1. Define the colours at each corner
    Const Corner000 As Long = &HFF0000    'red                  0
    Const Corner001 As Long = &HFF00       'green               1
    Const Corner010 As Long = &HFF           'blue                 2
    Const Corner011 As Long = &HFF00FF    'magenta           3
    Const Corner100 As Long = &HFFFF00    'yellow               4
    Const Corner101 As Long = &HFFFF       'cyan                 5
    Const Corner110 As Long = &HFF8000   'orange              6
    Const Corner111 As Long = &HFFFFFF       'white              7
    
    
    
'//2. Define the faces
    'top
        CubeVerts(0) = CreateLitVertex(-1, 1, -1, Corner010, 0, 0, 0)
        CubeVerts(1) = CreateLitVertex(1, 1, -1, Corner110, 0, 0, 0)
        CubeVerts(2) = CreateLitVertex(-1, 1, 1, Corner011, 0, 0, 0)
        
        CubeVerts(3) = CreateLitVertex(1, 1, -1, Corner110, 0, 0, 0)
        CubeVerts(4) = CreateLitVertex(1, 1, 1, Corner111, 0, 0, 0)
        CubeVerts(5) = CreateLitVertex(-1, 1, 1, Corner011, 0, 0, 0)
        
    'bottom
        CubeVerts(6) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 0)
        CubeVerts(7) = CreateLitVertex(1, -1, -1, Corner100, 0, 0, 0)
        CubeVerts(8) = CreateLitVertex(-1, -1, 1, Corner001, 0, 0, 0)
        
        CubeVerts(9) = CreateLitVertex(1, -1, -1, Corner100, 0, 0, 0)
        CubeVerts(10) = CreateLitVertex(1, -1, 1, Corner101, 0, 0, 0)
        CubeVerts(11) = CreateLitVertex(-1, -1, 1, Corner001, 0, 0, 0)
    
    'left
        CubeVerts(12) = CreateLitVertex(-1, 1, -1, Corner010, 0, 0, 0)
        CubeVerts(13) = CreateLitVertex(-1, 1, 1, Corner011, 0, 0, 0)
        CubeVerts(14) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 0)
        
        CubeVerts(15) = CreateLitVertex(-1, 1, 1, Corner011, 0, 0, 0)
        CubeVerts(16) = CreateLitVertex(-1, -1, 1, Corner001, 0, 0, 0)
        CubeVerts(17) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 0)
    
    'right
        CubeVerts(18) = CreateLitVertex(1, 1, -1, Corner110, 0, 0, 0)
        CubeVerts(19) = CreateLitVertex(1, 1, 1, Corner111, 0, 0, 0)
        CubeVerts(20) = CreateLitVertex(1, -1, -1, Corner100, 0, 0, 0)
        
        CubeVerts(21) = CreateLitVertex(1, 1, 1, Corner111, 0, 0, 0)
        CubeVerts(22) = CreateLitVertex(1, -1, 1, Corner101, 0, 0, 0)
        CubeVerts(23) = CreateLitVertex(1, -1, -1, Corner100, 0, 0, 0)
    
    'front
        CubeVerts(24) = CreateLitVertex(-1, 1, 1, Corner011, 0, 0, 0)
        CubeVerts(25) = CreateLitVertex(1, 1, 1, Corner111, 0, 0, 0)
        CubeVerts(26) = CreateLitVertex(-1, -1, 1, Corner001, 0, 0, 0)
        
        CubeVerts(27) = CreateLitVertex(1, 1, 1, Corner111, 0, 0, 0)
        CubeVerts(28) = CreateLitVertex(1, -1, 1, Corner101, 0, 0, 0)
        CubeVerts(29) = CreateLitVertex(-1, -1, 1, Corner001, 0, 0, 0)
    
    'back
        CubeVerts(30) = CreateLitVertex(-1, 1, -1, Corner010, 0, 0, 0)
        CubeVerts(31) = CreateLitVertex(1, 1, -1, Corner110, 0, 0, 0)
        CubeVerts(32) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 0)
        
        CubeVerts(33) = CreateLitVertex(1, 1, -1, Corner110, 0, 0, 0)
        CubeVerts(34) = CreateLitVertex(1, -1, -1, Corner100, 0, 0, 0)
        CubeVerts(35) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 0)
        
'//Setup the Vertex buffer
    Set vbCube = D3DDevice.CreateVertexBuffer(Len(CubeVerts(0)) * 36, 0, FVF_LVERTEX, D3DPOOL_MANAGED)
    If vbCube Is Nothing Then Debug.Print "ERROR: Could not create vertex buffer": Exit Sub
    
    D3DVertexBuffer8SetData vbCube, 0, Len(CubeVerts(0)) * 36, 0, CubeVerts(0)
    
'//Setup the Vertex buffer for the indices version...
    vList(0) = CreateLitVertex(-1, -1, -1, &HFFFFFF, 0, 0, 0)
    vList(1) = CreateLitVertex(-1, -1, 1, &HFF0000, 0, 0, 0)
    vList(2) = CreateLitVertex(-1, 1, -1, &HFF00, 0, 0, 0)
    vList(3) = CreateLitVertex(-1, 1, 1, &HFF, 0, 0, 0)
    vList(4) = CreateLitVertex(1, -1, -1, &HFF00FF, 0, 0, 0)
    vList(5) = CreateLitVertex(1, -1, 1, &HFFFF00, 0, 0, 0)
    vList(6) = CreateLitVertex(1, 1, -1, &HFFFF, 0, 0, 0)
    vList(7) = CreateLitVertex(1, 1, 1, &HFF8000, 0, 0, 0)
    
    Set vbCubeIdx = D3DDevice.CreateVertexBuffer(Len(vList(0)) * 8, 0, FVF_LVERTEX, D3DPOOL_MANAGED)
    If vbCubeIdx Is Nothing Then Debug.Print "ERROR: Could not create vbCubeIdx": Exit Sub
    
    D3DVertexBuffer8SetData vbCubeIdx, 0, Len(vList(0)) * 8, 0, vList(0)
    
'//Setup the index list.
    'top
    iList(0) = 2: iList(1) = 6: iList(2) = 3
    iList(3) = 6: iList(4) = 7: iList(5) = 3
    
    'bottom
    iList(6) = 0: iList(7) = 4: iList(8) = 1
    iList(9) = 4: iList(10) = 5: iList(11) = 1
    
    'left
    iList(12) = 2: iList(13) = 3: iList(14) = 0
    iList(15) = 3: iList(16) = 1: iList(17) = 0
    
    'right
    iList(18) = 6: iList(19) = 7: iList(20) = 4
    iList(21) = 7: iList(22) = 5: iList(23) = 4
    
    'front
    iList(24) = 3: iList(25) = 7: iList(26) = 1
    iList(27) = 7: iList(28) = 5: iList(29) = 1
    
    'back
    iList(30) = 2: iList(31) = 6: iList(32) = 0
    iList(33) = 6: iList(34) = 4: iList(35) = 0
    
'//Create the index buffer
    Set ibCube = D3DDevice.CreateIndexBuffer(Len(iList(0)) * 36, 0, D3DFMT_INDEX16, D3DPOOL_MANAGED)
    If ibCube Is Nothing Then Debug.Print "ERROR: Could not create the index buffer": Exit Sub
    
    D3DIndexBuffer8SetData ibCube, 0, Len(iList(0)) * 36, 0, iList(0)
    
'//A simple example of checking how many indices there are:
    'D3DFMT_INDEX16 = 101
    'D3DFMT_INDEX32 = 102
    Dim ibDesc As D3DINDEXBUFFER_DESC
    Dim IndexCount As Long 'how many indices are in the buffer
    
    ibCube.GetDesc ibDesc
    If ibDesc.Format = 101 Then
        '16 bit indices
        IndexCount = ibDesc.Size / 2
    ElseIf ibDesc.Format = 102 Then
        '32 bit indices
        IndexCount = ibDesc.Size / 4
    Else
        'no idea whats stored here!
    End If
    
    Debug.Print IndexCount, " indices in the buffer."

'//A simple example of checking how many vertices there are:
    Dim vbDesc As D3DVERTEXBUFFER_DESC
    Dim DummyVertex As LITVERTEX
    Dim VertexCount As Long
    Dim TotalDivider As Long 'the size of the vertex structure
    
    vbCube.GetDesc vbDesc
    
    If vbDesc.FVF = FVF_LVERTEX Then
        'The type stored is the LVertex type
        VertexCount = vbDesc.Size / Len(DummyVertex)
    Else
        'it's some other type of vertex, lets find out:
        If Not (vbDesc.FVF And D3DFVF_XYZ) = 0 Then
            Debug.Print "D3DFVF_XYZ"
            TotalDivider = TotalDivider + 12
        End If
        
        If Not (vbDesc.FVF And D3DFVF_XYZRHW) = 0 Then
            Debug.Print "D3DFVF_XYZRHW"
            TotalDivider = TotalDivider + 16
        End If
        
        If Not (vbDesc.FVF And D3DFVF_NORMAL) = 0 Then
            Debug.Print "D3DFVF_NORMAL"
            TotalDivider = TotalDivider + 12
        End If
        
        If Not (vbDesc.FVF And D3DFVF_DIFFUSE) = 0 Then
            Debug.Print "D3DFVF_DIFFUSE"
            TotalDivider = TotalDivider + 4
        End If
        
        If Not (vbDesc.FVF And D3DFVF_SPECULAR) = 0 Then
            Debug.Print "D3DFVF_SPECULAR"
            TotalDivider = TotalDivider + 4
        End If
        
        If Not (vbDesc.FVF And D3DFVF_TEX1) = 0 Then
            Debug.Print "D3DFVF_TEX1"
            TotalDivider = TotalDivider + 8
        End If
        
        If Not (vbDesc.FVF And D3DFVF_TEX2) = 0 Then
            Debug.Print "D3DFVF_TEX2"
            TotalDivider = TotalDivider + 8
        End If
        
        VertexCount = vbDesc.Size / TotalDivider
    End If
    
    Debug.Print VertexCount, " Vertices in the buffer"
    
End Sub

Private Function CreateLitVertex(X As Single, Y As Single, Z As Single, Colour As Long, Specular As Long, tU As Single, tV As Single) As LITVERTEX
    CreateLitVertex.X = X
    CreateLitVertex.Y = Y
    CreateLitVertex.Z = Z
    CreateLitVertex.Color = Colour
    CreateLitVertex.Specular = Specular
    CreateLitVertex.tU = tU
    CreateLitVertex.tV = tV
End Function

Private Function MakeVector(X As Single, Y As Single, Z As Single) As D3DVECTOR
    MakeVector.X = X: MakeVector.Y = Y: MakeVector.Z = Z
End Function

Private Function CheckDisplayMode(Width As Long, Height As Long, Depth As Long) As CONST_D3DFORMAT
'//0. any variables
    Dim I As Long
    Dim DispMode As D3DDISPLAYMODE
    
'//1. Scan through
    For I = 0 To D3D.GetAdapterModeCount(0) - 1
        D3D.EnumAdapterModes 0, I, DispMode
        If DispMode.Width = Width Then
            If DispMode.Height = Height Then
                If DispMode.Format = D3DFMT_R5G6B5 Or D3DFMT_X1R5G5B5 Or D3DFMT_X4R4G4B4 Then
                    '16 bit mode
                        If Depth = 16 Then
                            CheckDisplayMode = DispMode.Format
                            Exit Function
                        End If
                ElseIf DispMode.Format = D3DFMT_R8G8B8 Or D3DFMT_X8R8G8B8 Then
                    '32bit mode
                        If Depth = 32 Then
                            CheckDisplayMode = DispMode.Format
                            Exit Function
                        End If
                End If
            End If
        End If
    Next I
CheckDisplayMode = D3DFMT_UNKNOWN
End Function


Private Sub Form_Click()
bRunning = False
End Sub

Private Sub Form_Load()
Dim matTemp As D3DMATRIX
Dim Angle As Single

Me.Show '//Make sure our window is visible

bRunning = Initialise()
Debug.Print "Device Creation Return Code : ", bRunning 'So you can see what happens...

Do While bRunning
    D3DXMatrixIdentity matWorld
    
    D3DXMatrixIdentity matTemp
    D3DXMatrixRotationX matTemp, Angle * RAD
    D3DXMatrixMultiply matWorld, matWorld, matTemp
    
    D3DXMatrixIdentity matTemp
    D3DXMatrixRotationY matTemp, Angle * RAD
    D3DXMatrixMultiply matWorld, matWorld, matTemp
    
    D3DXMatrixIdentity matTemp
    D3DXMatrixRotationZ matTemp, Angle * RAD
    D3DXMatrixMultiply matWorld, matWorld, matTemp
    
    Angle = Angle + 1
    
    D3DDevice.SetTransform D3DTS_WORLD, matWorld
    
    Render '//Update the frame...
    DoEvents '//Allow windows time to think; otherwise you'll get into a really tight (and bad) loop...

Loop '//Begin the next frame...

'//If we've gotten to this point the loop must have been terminated
'   So we need to clean up after ourselves. This isn't essential, but it'
'   good coding practise.

On Error Resume Next 'If the objects were never created;
'                               (the initialisation failed) we might get an
'                               error when freeing them... which we need to
'                               handle, but as we're closing anyway...
Set D3DDevice = Nothing
Set D3D = Nothing
Set Dx = Nothing
Debug.Print "All Objects Destroyed"

'//Final termination:
Unload Me
End
End Sub

